@@ -4,9 +4,9 @@ imdb package.
 This package can be used to retrieve information about a movie or
 a person from the IMDb database.
 It can fetch data through different media (e.g.: the IMDb web pages,
-a local installation, a SQL database, etc.)
+a SQL database, etc.)
 
-Copyright 2004-2008 Davide Alberani <da@erlug.linux.it>
+Copyright 2004-2014 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,21 +25,28 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 __all__ = ['IMDb', 'IMDbError', 'Movie', 'Person', 'Character', 'Company',
             'available_access_systems']
-__version__ = VERSION = '3.6'
+__version__ = VERSION = '5.0'
 
 # Import compatibility module (importing it is enough).
 import _compat
 
-import sys, os, ConfigParser
-from types import UnicodeType, TupleType, ListType, MethodType
+import sys, os, ConfigParser, logging
+from types import MethodType
 
 from imdb import Movie, Person, Character, Company
-from imdb._exceptions import IMDbError, IMDbDataAccessError
+import imdb._logging
+from imdb._exceptions import IMDbError, IMDbDataAccessError, IMDbParserError
 from imdb.utils import build_title, build_name, build_company_name
 
+_aux_logger = logging.getLogger('imdbpy.aux')
+
 
 # URLs of the main pages for movies, persons, characters and queries.
 imdbURL_base = 'http://akas.imdb.com/'
+
+# NOTE: the urls below will be removed in a future version.
+#       please use the values in the 'urls' attribute
+#       of the IMDbBase subclass instance.
 # http://akas.imdb.com/title/
 imdbURL_movie_base = '%stitle/' % imdbURL_base
 # http://akas.imdb.com/title/tt%s/
@@ -56,6 +63,12 @@ imdbURL_character_main = imdbURL_character_base + 'ch%s/'
 imdbURL_company_base = '%scompany/' % imdbURL_base
 # http://akas.imdb.com/company/co%s/
 imdbURL_company_main = imdbURL_company_base + 'co%s/'
+# http://akas.imdb.com/keyword/%s/
+imdbURL_keyword_main = imdbURL_base + 'keyword/%s/'
+# http://akas.imdb.com/chart/top
+imdbURL_top250 = imdbURL_base + 'chart/top'
+# http://akas.imdb.com/chart/bottom
+imdbURL_bottom100 = imdbURL_base + 'chart/bottom'
 # http://akas.imdb.com/find?%s
 imdbURL_find = imdbURL_base + 'find?%s'
 
@@ -91,8 +104,7 @@ class ConfigParserWithCase(ConfigParser.ConfigParser):
                 self.read(fname)
             except (ConfigParser.MissingSectionHeaderError,
                     ConfigParser.ParsingError), e:
-                import warnings
-                warnings.warn('Troubles reading config file: %s' % e)
+                _aux_logger.warn('Troubles reading config file: %s' % e)
             # Stop at the first valid file.
             if self.has_section('imdbpy'):
                 break
@@ -148,38 +160,47 @@ def IMDb(accessSystem=None, *arguments, **keywords):
             kwds.update(keywords)
             keywords = kwds
         except Exception, e:
-            import warnings
-            warnings.warn('Unable to read configuration file; ' + \
-                            'complete error: %s' % e)
+            import logging
+            logging.getLogger('imdbpy').warn('Unable to read configuration' \
+                                            ' file; complete error: %s' % e)
             # It just LOOKS LIKE a bad habit: we tried to read config
             # options from some files, but something is gone horribly
             # wrong: ignore everything and pretend we were called with
             # the 'http' accessSystem.
             accessSystem = 'http'
+    if 'loggingLevel' in keywords:
+        imdb._logging.setLevel(keywords['loggingLevel'])
+        del keywords['loggingLevel']
+    if 'loggingConfig' in keywords:
+        logCfg = keywords['loggingConfig']
+        del keywords['loggingConfig']
+        try:
+            import logging.config
+            logging.config.fileConfig(os.path.expanduser(logCfg))
+        except Exception, e:
+            logging.getLogger('imdbpy').warn('unable to read logger ' \
+                                            'config: %s' % e)
+    if accessSystem in ('httpThin', 'webThin', 'htmlThin'):
+        logging.warn('httpThin was removed since IMDbPY 4.8')
+        accessSystem = 'http'
     if accessSystem in ('http', 'web', 'html'):
         from parser.http import IMDbHTTPAccessSystem
         return IMDbHTTPAccessSystem(*arguments, **keywords)
-    elif accessSystem in ('httpThin', 'webThin', 'htmlThin'):
-        from parser.http import IMDbHTTPAccessSystem
-        return IMDbHTTPAccessSystem(isThin=1, *arguments, **keywords)
     elif accessSystem in ('mobile',):
         from parser.mobile import IMDbMobileAccessSystem
         return IMDbMobileAccessSystem(*arguments, **keywords)
     elif accessSystem in ('local', 'files'):
-        try:
-            from parser.local import IMDbLocalAccessSystem
-        except ImportError:
-            raise IMDbError, 'the local access system is not installed'
-        return IMDbLocalAccessSystem(*arguments, **keywords)
+        # The local access system was removed since IMDbPY 4.2.
+        raise IMDbError('the local access system was removed since IMDbPY 4.2')
     elif accessSystem in ('sql', 'db', 'database'):
         try:
             from parser.sql import IMDbSqlAccessSystem
         except ImportError:
-            raise IMDbError, 'the sql access system is not installed'
+            raise IMDbError('the sql access system is not installed')
         return IMDbSqlAccessSystem(*arguments, **keywords)
     else:
-        raise IMDbError, 'unknown kind of data access system: "%s"' \
-                            % accessSystem
+        raise IMDbError('unknown kind of data access system: "%s"' \
+                            % accessSystem)
 
 
 def available_access_systems():
@@ -188,7 +209,7 @@ def available_access_systems():
     # XXX: trying to import modules is a good thing?
     try:
         from parser.http import IMDbHTTPAccessSystem
-        asList += ['http', 'httpThin']
+        asList.append('http')
     except ImportError:
         pass
     try:
@@ -196,11 +217,6 @@ def available_access_systems():
         asList.append('mobile')
     except ImportError:
         pass
-    try:
-        from parser.local import IMDbLocalAccessSystem
-        asList.append('local')
-    except ImportError:
-        pass
     try:
         from parser.sql import IMDbSqlAccessSystem
         asList.append('sql')
@@ -228,7 +244,13 @@ class IMDbBase:
     # in the subclasses).
     accessSystem = 'UNKNOWN'
 
-    def __init__(self, defaultModFunct=None, results=20,
+    # Top-level logger for IMDbPY.
+    _imdb_logger = logging.getLogger('imdbpy')
+
+    # Whether to re-raise caught exceptions or not.
+    _reraise_exceptions = False
+
+    def __init__(self, defaultModFunct=None, results=20, keywordsResults=100,
                 *arguments, **keywords):
         """Initialize the access system.
         If specified, defaultModFunct is the function used by
@@ -246,6 +268,60 @@ class IMDbBase:
         if results < 1:
             results = 20
         self._results = results
+        try:
+            keywordsResults = int(keywordsResults)
+        except (TypeError, ValueError):
+            keywordsResults = 100
+        if keywordsResults < 1:
+            keywordsResults = 100
+        self._keywordsResults = keywordsResults
+        self._reraise_exceptions = keywords.get('reraiseExceptions') or False
+        self.set_imdb_urls(keywords.get('imdbURL_base') or imdbURL_base)
+
+    def set_imdb_urls(self, imdbURL_base):
+        """Set the urls used accessing the IMDb site."""
+        imdbURL_base = imdbURL_base.strip().strip('"\'')
+        if not imdbURL_base.startswith('http://'):
+            imdbURL_base = 'http://%s' % imdbURL_base
+        if not imdbURL_base.endswith('/'):
+            imdbURL_base = '%s/' % imdbURL_base
+        # http://akas.imdb.com/title/
+        imdbURL_movie_base='%stitle/' % imdbURL_base
+        # http://akas.imdb.com/title/tt%s/
+        imdbURL_movie_main=imdbURL_movie_base + 'tt%s/'
+        # http://akas.imdb.com/name/
+        imdbURL_person_base='%sname/' % imdbURL_base
+        # http://akas.imdb.com/name/nm%s/
+        imdbURL_person_main=imdbURL_person_base + 'nm%s/'
+        # http://akas.imdb.com/character/
+        imdbURL_character_base='%scharacter/' % imdbURL_base
+        # http://akas.imdb.com/character/ch%s/
+        imdbURL_character_main=imdbURL_character_base + 'ch%s/'
+        # http://akas.imdb.com/company/
+        imdbURL_company_base='%scompany/' % imdbURL_base
+        # http://akas.imdb.com/company/co%s/
+        imdbURL_company_main=imdbURL_company_base + 'co%s/'
+        # http://akas.imdb.com/keyword/%s/
+        imdbURL_keyword_main=imdbURL_base + 'keyword/%s/'
+        # http://akas.imdb.com/chart/top
+        imdbURL_top250=imdbURL_base + 'chart/top'
+        # http://akas.imdb.com/chart/bottom
+        imdbURL_bottom100=imdbURL_base + 'chart/bottom'
+        # http://akas.imdb.com/find?%s
+        imdbURL_find=imdbURL_base + 'find?%s'
+        self.urls = dict(
+            movie_base=imdbURL_movie_base,
+            movie_main=imdbURL_movie_main,
+            person_base=imdbURL_person_base,
+            person_main=imdbURL_person_main,
+            character_base=imdbURL_character_base,
+            character_main=imdbURL_character_main,
+            company_base=imdbURL_company_base,
+            company_main=imdbURL_company_main,
+            keyword_main=imdbURL_keyword_main,
+            top250=imdbURL_top250,
+            bottom100=imdbURL_bottom100,
+            find=imdbURL_find)
 
     def _normalize_movieID(self, movieID):
         """Normalize the given movieID."""
@@ -335,13 +411,15 @@ class IMDbBase:
         self.update(movie, info)
         return movie
 
+    get_episode = get_movie
+
     def _search_movie(self, title, results):
         """Return a list of tuples (movieID, {movieData})"""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
-    def search_movie(self, title, results=None):
+    def search_movie(self, title, results=None, _episodes=False):
         """Return a list of Movie objects for a query for the given title.
         The results argument is the maximum number of results to return."""
         if results is None:
@@ -352,13 +430,28 @@ class IMDbBase:
             results = 20
         # XXX: I suppose it will be much safer if the user provides
         #      an unicode string... this is just a guess.
-        if not isinstance(title, UnicodeType):
+        if not isinstance(title, unicode):
             title = unicode(title, encoding, 'replace')
-        res = self._search_movie(title, results)
+        if not _episodes:
+            res = self._search_movie(title, results)
+        else:
+            res = self._search_episode(title, results)
         return [Movie.Movie(movieID=self._get_real_movieID(mi),
                 data=md, modFunct=self._defModFunct,
                 accessSystem=self.accessSystem) for mi, md in res][:results]
 
+    def _search_episode(self, title, results):
+        """Return a list of tuples (movieID, {movieData})"""
+        # XXX: for the real implementation, see the method of the
+        #      subclass, somewhere under the imdb.parser package.
+        raise NotImplementedError('override this method')
+
+    def search_episode(self, title, results=None):
+        """Return a list of Movie objects for a query for the given title.
+        The results argument is the maximum number of results to return;
+        this method searches only for titles of tv (mini) series' episodes."""
+        return self.search_movie(title, results=results, _episodes=True)
+
     def get_person(self, personID, info=Person.Person.default_info,
                     modFunct=None):
         """Return a Person object for the given personID.
@@ -385,7 +478,7 @@ class IMDbBase:
         """Return a list of tuples (personID, {personData})"""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def search_person(self, name, results=None):
         """Return a list of Person objects for a query for the given name.
@@ -397,7 +490,7 @@ class IMDbBase:
             results = int(results)
         except (ValueError, OverflowError):
             results = 20
-        if not isinstance(name, UnicodeType):
+        if not isinstance(name, unicode):
             name = unicode(name, encoding, 'replace')
         res = self._search_person(name, results)
         return [Person.Person(personID=self._get_real_personID(pi),
@@ -430,7 +523,7 @@ class IMDbBase:
         """Return a list of tuples (characterID, {characterData})"""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def search_character(self, name, results=None):
         """Return a list of Character objects for a query for the given name.
@@ -442,7 +535,7 @@ class IMDbBase:
             results = int(results)
         except (ValueError, OverflowError):
             results = 20
-        if not isinstance(name, UnicodeType):
+        if not isinstance(name, unicode):
             name = unicode(name, encoding, 'replace')
         res = self._search_character(name, results)
         return [Character.Character(characterID=self._get_real_characterID(pi),
@@ -475,7 +568,7 @@ class IMDbBase:
         """Return a list of tuples (companyID, {companyData})"""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def search_company(self, name, results=None):
         """Return a list of Company objects for a query for the given name.
@@ -487,22 +580,85 @@ class IMDbBase:
             results = int(results)
         except (ValueError, OverflowError):
             results = 20
-        if not isinstance(name, UnicodeType):
+        if not isinstance(name, unicode):
             name = unicode(name, encoding, 'replace')
         res = self._search_company(name, results)
         return [Company.Company(companyID=self._get_real_companyID(pi),
                 data=pd, modFunct=self._defModFunct,
                 accessSystem=self.accessSystem) for pi, pd in res][:results]
 
+    def _search_keyword(self, keyword, results):
+        """Return a list of 'keyword' strings."""
+        # XXX: for the real implementation, see the method of the
+        #      subclass, somewhere under the imdb.parser package.
+        raise NotImplementedError('override this method')
+
+    def search_keyword(self, keyword, results=None):
+        """Search for existing keywords, similar to the given one."""
+        if results is None:
+            results = self._keywordsResults
+        try:
+            results = int(results)
+        except (ValueError, OverflowError):
+            results = 100
+        if not isinstance(keyword, unicode):
+            keyword = unicode(keyword, encoding, 'replace')
+        return self._search_keyword(keyword, results)
+
+    def _get_keyword(self, keyword, results):
+        """Return a list of tuples (movieID, {movieData})"""
+        # XXX: for the real implementation, see the method of the
+        #      subclass, somewhere under the imdb.parser package.
+        raise NotImplementedError('override this method')
+
+    def get_keyword(self, keyword, results=None):
+        """Return a list of movies for the given keyword."""
+        if results is None:
+            results = self._keywordsResults
+        try:
+            results = int(results)
+        except (ValueError, OverflowError):
+            results = 100
+        # XXX: I suppose it will be much safer if the user provides
+        #      an unicode string... this is just a guess.
+        if not isinstance(keyword, unicode):
+            keyword = unicode(keyword, encoding, 'replace')
+        res = self._get_keyword(keyword, results)
+        return [Movie.Movie(movieID=self._get_real_movieID(mi),
+                data=md, modFunct=self._defModFunct,
+                accessSystem=self.accessSystem) for mi, md in res][:results]
+
+    def _get_top_bottom_movies(self, kind):
+        """Return the list of the top 250 or bottom 100 movies."""
+        # XXX: for the real implementation, see the method of the
+        #      subclass, somewhere under the imdb.parser package.
+        #      This method must return a list of (movieID, {movieDict})
+        #      tuples.  The kind parameter can be 'top' or 'bottom'.
+        raise NotImplementedError('override this method')
+
+    def get_top250_movies(self):
+        """Return the list of the top 250 movies."""
+        res = self._get_top_bottom_movies('top')
+        return [Movie.Movie(movieID=self._get_real_movieID(mi),
+                data=md, modFunct=self._defModFunct,
+                accessSystem=self.accessSystem) for mi, md in res]
+
+    def get_bottom100_movies(self):
+        """Return the list of the bottom 100 movies."""
+        res = self._get_top_bottom_movies('bottom')
+        return [Movie.Movie(movieID=self._get_real_movieID(mi),
+                data=md, modFunct=self._defModFunct,
+                accessSystem=self.accessSystem) for mi, md in res]
+
     def new_movie(self, *arguments, **keywords):
         """Return a Movie object."""
         # XXX: not really useful...
-        if keywords.has_key('title'):
-            if not isinstance(keywords['title'], UnicodeType):
+        if 'title' in keywords:
+            if not isinstance(keywords['title'], unicode):
                 keywords['title'] = unicode(keywords['title'],
                                             encoding, 'replace')
         elif len(arguments) > 1:
-            if not isinstance(arguments[1], UnicodeType):
+            if not isinstance(arguments[1], unicode):
                 arguments[1] = unicode(arguments[1], encoding, 'replace')
         return Movie.Movie(accessSystem=self.accessSystem,
                             *arguments, **keywords)
@@ -510,12 +666,12 @@ class IMDbBase:
     def new_person(self, *arguments, **keywords):
         """Return a Person object."""
         # XXX: not really useful...
-        if keywords.has_key('name'):
-            if not isinstance(keywords['name'], UnicodeType):
+        if 'name' in keywords:
+            if not isinstance(keywords['name'], unicode):
                 keywords['name'] = unicode(keywords['name'],
                                             encoding, 'replace')
         elif len(arguments) > 1:
-            if not isinstance(arguments[1], UnicodeType):
+            if not isinstance(arguments[1], unicode):
                 arguments[1] = unicode(arguments[1], encoding, 'replace')
         return Person.Person(accessSystem=self.accessSystem,
                                 *arguments, **keywords)
@@ -523,12 +679,12 @@ class IMDbBase:
     def new_character(self, *arguments, **keywords):
         """Return a Character object."""
         # XXX: not really useful...
-        if keywords.has_key('name'):
-            if not isinstance(keywords['name'], UnicodeType):
+        if 'name' in keywords:
+            if not isinstance(keywords['name'], unicode):
                 keywords['name'] = unicode(keywords['name'],
                                             encoding, 'replace')
         elif len(arguments) > 1:
-            if not isinstance(arguments[1], UnicodeType):
+            if not isinstance(arguments[1], unicode):
                 arguments[1] = unicode(arguments[1], encoding, 'replace')
         return Character.Character(accessSystem=self.accessSystem,
                                     *arguments, **keywords)
@@ -536,12 +692,12 @@ class IMDbBase:
     def new_company(self, *arguments, **keywords):
         """Return a Company object."""
         # XXX: not really useful...
-        if keywords.has_key('name'):
-            if not isinstance(keywords['name'], UnicodeType):
+        if 'name' in keywords:
+            if not isinstance(keywords['name'], unicode):
                 keywords['name'] = unicode(keywords['name'],
                                             encoding, 'replace')
         elif len(arguments) > 1:
-            if not isinstance(arguments[1], UnicodeType):
+            if not isinstance(arguments[1], unicode):
                 arguments[1] = unicode(arguments[1], encoding, 'replace')
         return Company.Company(accessSystem=self.accessSystem,
                                     *arguments, **keywords)
@@ -571,16 +727,16 @@ class IMDbBase:
             mopID = mop.companyID
             prefix = 'company'
         else:
-            raise IMDbError, 'object ' + repr(mop) + \
-                    ' is not a Movie, Person, Character or Company instance'
+            raise IMDbError('object ' + repr(mop) + \
+                    ' is not a Movie, Person, Character or Company instance')
         if mopID is None:
             # XXX: enough?  It's obvious that there are Characters
             #      objects without characterID, so I think they should
             #      just do nothing, when an i.update(character) is tried.
             if prefix == 'character':
                 return
-            raise IMDbDataAccessError, \
-                'the supplied object has null movieID, personID or companyID'
+            raise IMDbDataAccessError( \
+                'the supplied object has null movieID, personID or companyID')
         if mop.accessSystem == self.accessSystem:
             aSystem = self
         else:
@@ -596,29 +752,48 @@ class IMDbBase:
                 info = self.get_character_infoset()
             else:
                 info = self.get_company_infoset()
-        if not isinstance(info, (TupleType, ListType)):
+        if not isinstance(info, (tuple, list)):
             info = (info,)
         res = {}
         for i in info:
-            if i in mop.current_info and not override: continue
+            if i in mop.current_info and not override:
+                continue
+            if not i:
+                continue
+            self._imdb_logger.debug('retrieving "%s" info set', i)
             try:
                 method = getattr(aSystem, 'get_%s_%s' %
                                     (prefix, i.replace(' ', '_')))
             except AttributeError:
-                raise IMDbDataAccessError, 'unknown information set "%s"' % i
-            ret = method(mopID)
-            if ret.has_key('info sets'):
-                for ri in ret['info sets']:
-                    mop.add_to_current_info(ri)
-            else:
-                mop.add_to_current_info(i)
-            if ret.has_key('data'):
+                self._imdb_logger.error('unknown information set "%s"', i)
+                # Keeps going.
+                method = lambda *x: {}
+            try:
+                ret = method(mopID)
+            except Exception, e:
+                self._imdb_logger.critical('caught an exception retrieving ' \
+                                    'or parsing "%s" info set for mopID ' \
+                                    '"%s" (accessSystem: %s)',
+                                    i, mopID, mop.accessSystem, exc_info=True)
+                ret = {}
+                # If requested by the user, reraise the exception.
+                if self._reraise_exceptions:
+                    raise
+            keys = None
+            if 'data' in ret:
                 res.update(ret['data'])
-            if ret.has_key('titlesRefs'):
+                if isinstance(ret['data'], dict):
+                    keys = ret['data'].keys()
+            if 'info sets' in ret:
+                for ri in ret['info sets']:
+                    mop.add_to_current_info(ri, keys, mainInfoset=i)
+            else:
+                mop.add_to_current_info(i, keys)
+            if 'titlesRefs' in ret:
                 mop.update_titlesRefs(ret['titlesRefs'])
-            if ret.has_key('namesRefs'):
+            if 'namesRefs' in ret:
                 mop.update_namesRefs(ret['namesRefs'])
-            if ret.has_key('charactersRefs'):
+            if 'charactersRefs' in ret:
                 mop.update_charactersRefs(ret['charactersRefs'])
         mop.set_data(res, override=0)
 
@@ -627,157 +802,106 @@ class IMDbBase:
         web server); must be overridden by the subclass."""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def get_imdbPersonID(self, personID):
         """Translate a personID in a imdbID (the ID used by the IMDb
         web server); must be overridden by the subclass."""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def get_imdbCharacterID(self, characterID):
         """Translate a characterID in a imdbID (the ID used by the IMDb
         web server); must be overridden by the subclass."""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def get_imdbCompanyID(self, companyID):
         """Translate a companyID in a imdbID (the ID used by the IMDb
         web server); must be overridden by the subclass."""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
-    def _searchIMDb(self, params):
-        """Fetch the given search page from the IMDb akas server."""
-        from imdb.parser.http import IMDbURLopener
-        url = imdbURL_find % params
-        content = u''
+    def _searchIMDb(self, kind, ton, title_kind=None):
+        """Search the IMDb akas server for the given title or name."""
+        # The Exact Primary search system has gone AWOL, so we resort
+        # to the mobile search. :-/
+        if not ton:
+            return None
+        ton = ton.strip('"')
+        aSystem = IMDb('mobile')
+        if kind == 'tt':
+            searchFunct = aSystem.search_movie
+            check = 'long imdb title'
+        elif kind == 'nm':
+            searchFunct = aSystem.search_person
+            check = 'long imdb name'
+        elif kind == 'char':
+            searchFunct = aSystem.search_character
+            check = 'long imdb name'
+        elif kind == 'co':
+            # XXX: are [COUNTRY] codes included in the results?
+            searchFunct = aSystem.search_company
+            check = 'long imdb name'
         try:
-            urlOpener = IMDbURLopener()
-            content = urlOpener.retrieve_unicode(url)
-        except (IOError, IMDbDataAccessError):
-            pass
-        return content
+            searchRes = searchFunct(ton)
+        except IMDbError:
+            return None
+        # When only one result is returned, assume it was from an
+        # exact match.
+        if len(searchRes) == 1:
+            return searchRes[0].getID()
+        title_only_matches = []
+        for item in searchRes:
+            # Return the first perfect match.
+            if item[check].strip('"') == ton:
+                # For titles do additional check for kind
+                if kind != 'tt' or title_kind == item['kind']:
+                    return item.getID()
+                elif kind == 'tt':
+                    title_only_matches.append(item.getID())
+        # imdbpy2sql.py could detected wrong type, so if no title and kind
+        # matches found - collect all results with title only match
+        # Return list of IDs if multiple matches (can happen when searching
+        # titles with no title_kind specified)
+        # Example: DB: Band of Brothers "tv series" vs "tv mini-series"
+        if title_only_matches:
+            if len(title_only_matches) == 1:
+                return title_only_matches[0]
+            else:
+                return title_only_matches
+        return None
 
-    def title2imdbID(self, title):
+    def title2imdbID(self, title, kind=None):
         """Translate a movie title (in the plain text data files format)
         to an imdbID.
         Try an Exact Primary Title search on IMDb;
-        return None if it's unable to get the imdbID."""
-        if not title: return None
-        import urllib
-        if isinstance(title, unicode):
-            title = title.encode('utf-8')
-        params = 'q=%s;s=pt' % str(urllib.quote_plus(title))
-        content = self._searchIMDb(params)
-        if content and content[:512].find('<title>IMDb Title') != -1:
-            # Sometimes (e.g.: for titles with a "+" in it) a list
-            # of results is returned even for Exact Primary searches;
-            # this try to deal with it, hoping that a "normal" query
-            # will result in just on title.
-            params = 's=tt&q=%s' % str(urllib.quote_plus(title))
-            content = self._searchIMDb(params)
-        if not content: return None
-        from imdb.parser.http.searchMovieParser import BasicMovieParser
-        mparser = BasicMovieParser()
-        result = mparser.parse(content)
-        if not (result and result.get('data')): return None
-        return result['data'][0][0]
+        return None if it's unable to get the imdbID;
+        Always specify kind: movie, tv series, video game etc. or search can
+        return list of IDs if multiple matches found
+        """
+        return self._searchIMDb('tt', title, kind)
 
     def name2imdbID(self, name):
         """Translate a person name in an imdbID.
         Try an Exact Primary Name search on IMDb;
         return None if it's unable to get the imdbID."""
-        if not name: return None
-        import urllib
-        if isinstance(name, unicode):
-            name = name.encode('utf-8')
-        params = 'q=%s;s=pn' % str(urllib.quote_plus(name))
-        content = self._searchIMDb(params)
-        if content and content[:512].find('<title>IMDb Name') != -1:
-            params = 's=nm&q=%s' % str(urllib.quote_plus(name))
-            content = self._searchIMDb(params)
-        if not content: return None
-        from imdb.parser.http.searchPersonParser import BasicPersonParser
-        pparser = BasicPersonParser()
-        result = pparser.parse(content)
-        if not (result and result.get('data')): return None
-        return result['data'][0][0]
+        return self._searchIMDb('nm', name)
 
     def character2imdbID(self, name):
         """Translate a character name in an imdbID.
         Try an Exact Primary Name search on IMDb;
         return None if it's unable to get the imdbID."""
-        if not name: return None
-        import urllib
-        if isinstance(name, unicode):
-            name = name.encode('utf-8')
-        # XXX: s=pc is just a assumption of mine; it doesn't seem
-        #      to work, and I can't find a working alternative.
-        params = 'q=%s;s=pc' % str(urllib.quote_plus(name))
-        content = self._searchIMDb(params)
-        if not content: return None
-        if content[:512].find('<title>IMDb  Search') != -1:
-            from imdb.parser.http.searchCharacterParser \
-                        import HTMLSearchCharacterParser, BasicCharacterParser
-            search_character_parser = HTMLSearchCharacterParser()
-            search_character_parser.kind = 'character'
-            search_character_parser._basic_parser = BasicCharacterParser
-            result = search_character_parser.parse(content)
-            if not result: return None
-            if not result.has_key('data'): return None
-            if not result['data']: return None
-            # Try to read the first result.
-            chID, dname = result['data'][0]
-            if isinstance(dname.get('name'), unicode):
-                rname = dname['name'].encode('utf-8')
-            if name == rname:
-                return chID
-            return None
-        from imdb.parser.http.searchCharacterParser import BasicCharacterParser
-        cparser = BasicCharacterParser()
-        result = cparser.parse(content)
-        if not (result and result.get('data')): return None
-        return result['data'][0][0]
+        return self._searchIMDb('char', name)
 
     def company2imdbID(self, name):
         """Translate a company name in an imdbID.
         Try an Exact Primary Name search on IMDb;
         return None if it's unable to get the imdbID."""
-        if not name: return None
-        import urllib
-        if isinstance(name, unicode):
-            name = name.encode('utf-8')
-        # XXX: s=pco is just a assumption of mine; it doesn't seem
-        #      to work, and I can't find a working alternative.
-        params = 'q=%s;s=pco' % str(urllib.quote_plus(name))
-        content = self._searchIMDb(params)
-        if not content: return None
-        if content[:512].find('<title>IMDb  Search') != -1:
-            from imdb.parser.http.searchCompanyParser \
-                        import HTMLSearchCompanyParser, BasicCompanyParser
-            search_company_parser = HTMLSearchCompanyParser()
-            search_company_parser.kind = 'company'
-            search_company_parser._basic_parser = BasicCompanyParser
-            result = search_company_parser.parse(content)
-            if not result: return None
-            if not result.has_key('data'): return None
-            if not result['data']: return None
-            # Try to read the first result.
-            chID, dname = result['data'][0]
-            if isinstance(dname.get('name'), unicode):
-                rname = dname['name'].encode('utf-8')
-            if name == rname:
-                return chID
-            return None
-        from imdb.parser.http.searchCompanyParser import BasicCompanyParser
-        cparser = BasicCompanyParser()
-        result = cparser.parse(content)
-        if not (result and result.get('data')): return None
-        return result['data'][0][0]
+        return self._searchIMDb('co', name)
 
     def get_imdbID(self, mop):
         """Return the imdbID for the given Movie, Person, Character or Company
@@ -791,8 +915,9 @@ class IMDbBase:
             if mop.movieID is not None:
                 imdbID = aSystem.get_imdbMovieID(mop.movieID)
             else:
-                imdbID = aSystem.title2imdbID(build_title(mop, canonical=1,
-                                                ptdf=1))
+                imdbID = aSystem.title2imdbID(build_title(mop, canonical=0,
+                                                ptdf=0, appendKind=False),
+                                                mop['kind'])
         elif isinstance(mop, Person.Person):
             if mop.personID is not None:
                 imdbID = aSystem.get_imdbPersonID(mop.personID)
@@ -802,6 +927,7 @@ class IMDbBase:
             if mop.characterID is not None:
                 imdbID = aSystem.get_imdbCharacterID(mop.characterID)
             else:
+                # canonical=0 ?
                 imdbID = aSystem.character2imdbID(build_name(mop, canonical=1))
         elif isinstance(mop, Company.Company):
             if mop.companyID is not None:
@@ -809,8 +935,8 @@ class IMDbBase:
             else:
                 imdbID = aSystem.company2imdbID(build_company_name(mop))
         else:
-            raise IMDbError, 'object ' + repr(mop) + \
-                        ' is not a Movie, Person or Character instance'
+            raise IMDbError('object ' + repr(mop) + \
+                        ' is not a Movie, Person or Character instance')
         return imdbID
 
     def get_imdbURL(self, mop):
@@ -828,8 +954,8 @@ class IMDbBase:
         elif isinstance(mop, Company.Company):
             url_firstPart = imdbURL_company_main
         else:
-            raise IMDbError, 'object ' + repr(mop) + \
-                        ' is not a Movie, Person, Character or Company instance'
+            raise IMDbError('object ' + repr(mop) + \
+                        ' is not a Movie, Person, Character or Company instance')
         return url_firstPart % imdbID
 
     def get_special_methods(self):
